from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class VitaminAUnits(Enum):
        """
            VitaminAUnits enumeration
        """
        
        InternationalUnit = 'international_unit'
        """
            
        """
        

class VitaminA(AbstractMeasure):
    """
    Vitamin A: 1 IU is the biological equivalent of 0.3 µg retinol, or of 0.6 µg beta-carotene.

    Args:
        value (float): The value.
        from_unit (VitaminAUnits): The VitaminA unit to create from, The default unit is InternationalUnit
    """
    def __init__(self, value: float, from_unit: VitaminAUnits = VitaminAUnits.InternationalUnit):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__international_units = None
        

    def convert(self, unit: VitaminAUnits) -> float:
        return self.__convert_from_base(unit)

    def __convert_from_base(self, from_unit: VitaminAUnits) -> float:
        value = self._value
        
        if from_unit == VitaminAUnits.InternationalUnit:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: VitaminAUnits) -> float:
        
        if to_unit == VitaminAUnits.InternationalUnit:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_international_units(international_units: float):
        """
        Create a new instance of VitaminA from a value in international_units.

        

        :param meters: The VitaminA value in international_units.
        :type international_units: float
        :return: A new instance of VitaminA.
        :rtype: VitaminA
        """
        return VitaminA(international_units, VitaminAUnits.InternationalUnit)

    
    @property
    def international_units(self) -> float:
        """
        
        """
        if self.__international_units != None:
            return self.__international_units
        self.__international_units = self.__convert_from_base(VitaminAUnits.InternationalUnit)
        return self.__international_units

    
    def to_string(self, unit: VitaminAUnits = VitaminAUnits.InternationalUnit) -> str:
        """
        Format the VitaminA to string.
        Note! the default format for VitaminA is InternationalUnit.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == VitaminAUnits.InternationalUnit:
            return f"""{self.international_units} IU"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: VitaminAUnits = VitaminAUnits.InternationalUnit) -> str:
        """
        Get VitaminA unit abbreviation.
        Note! the default abbreviation for VitaminA is InternationalUnit.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == VitaminAUnits.InternationalUnit:
            return """IU"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricChargeDensityUnits(Enum):
        """
            ElectricChargeDensityUnits enumeration
        """
        
        CoulombPerCubicMeter = 'coulomb_per_cubic_meter'
        """
            
        """
        

class ElectricChargeDensity(AbstractMeasure):
    """
    In electromagnetism, charge density is a measure of the amount of electric charge per volume.

    Args:
        value (float): The value.
        from_unit (ElectricChargeDensityUnits): The ElectricChargeDensity unit to create from, The default unit is CoulombPerCubicMeter
    """
    def __init__(self, value: float, from_unit: ElectricChargeDensityUnits = ElectricChargeDensityUnits.CoulombPerCubicMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__coulombs_per_cubic_meter = None
        

    def convert(self, unit: ElectricChargeDensityUnits) -> float:
        return self.__convert_from_base(unit)

    def __convert_from_base(self, from_unit: ElectricChargeDensityUnits) -> float:
        value = self._value
        
        if from_unit == ElectricChargeDensityUnits.CoulombPerCubicMeter:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricChargeDensityUnits) -> float:
        
        if to_unit == ElectricChargeDensityUnits.CoulombPerCubicMeter:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_coulombs_per_cubic_meter(coulombs_per_cubic_meter: float):
        """
        Create a new instance of ElectricChargeDensity from a value in coulombs_per_cubic_meter.

        

        :param meters: The ElectricChargeDensity value in coulombs_per_cubic_meter.
        :type coulombs_per_cubic_meter: float
        :return: A new instance of ElectricChargeDensity.
        :rtype: ElectricChargeDensity
        """
        return ElectricChargeDensity(coulombs_per_cubic_meter, ElectricChargeDensityUnits.CoulombPerCubicMeter)

    
    @property
    def coulombs_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__coulombs_per_cubic_meter != None:
            return self.__coulombs_per_cubic_meter
        self.__coulombs_per_cubic_meter = self.__convert_from_base(ElectricChargeDensityUnits.CoulombPerCubicMeter)
        return self.__coulombs_per_cubic_meter

    
    def to_string(self, unit: ElectricChargeDensityUnits = ElectricChargeDensityUnits.CoulombPerCubicMeter) -> str:
        """
        Format the ElectricChargeDensity to string.
        Note! the default format for ElectricChargeDensity is CoulombPerCubicMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ElectricChargeDensityUnits.CoulombPerCubicMeter:
            return f"""{self.coulombs_per_cubic_meter} C/m³"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricChargeDensityUnits = ElectricChargeDensityUnits.CoulombPerCubicMeter) -> str:
        """
        Get ElectricChargeDensity unit abbreviation.
        Note! the default abbreviation for ElectricChargeDensity is CoulombPerCubicMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricChargeDensityUnits.CoulombPerCubicMeter:
            return """C/m³"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class LuminousIntensityUnits(Enum):
        """
            LuminousIntensityUnits enumeration
        """
        
        Candela = 'candela'
        """
            
        """
        

class LuminousIntensity(AbstractMeasure):
    """
    In photometry, luminous intensity is a measure of the wavelength-weighted power emitted by a light source in a particular direction per unit solid angle, based on the luminosity function, a standardized model of the sensitivity of the human eye.

    Args:
        value (float): The value.
        from_unit (LuminousIntensityUnits): The LuminousIntensity unit to create from, The default unit is Candela
    """
    def __init__(self, value: float, from_unit: LuminousIntensityUnits = LuminousIntensityUnits.Candela):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__candela = None
        

    def convert(self, unit: LuminousIntensityUnits) -> float:
        return self.__convert_from_base(unit)

    def __convert_from_base(self, from_unit: LuminousIntensityUnits) -> float:
        value = self._value
        
        if from_unit == LuminousIntensityUnits.Candela:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: LuminousIntensityUnits) -> float:
        
        if to_unit == LuminousIntensityUnits.Candela:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_candela(candela: float):
        """
        Create a new instance of LuminousIntensity from a value in candela.

        

        :param meters: The LuminousIntensity value in candela.
        :type candela: float
        :return: A new instance of LuminousIntensity.
        :rtype: LuminousIntensity
        """
        return LuminousIntensity(candela, LuminousIntensityUnits.Candela)

    
    @property
    def candela(self) -> float:
        """
        
        """
        if self.__candela != None:
            return self.__candela
        self.__candela = self.__convert_from_base(LuminousIntensityUnits.Candela)
        return self.__candela

    
    def to_string(self, unit: LuminousIntensityUnits = LuminousIntensityUnits.Candela) -> str:
        """
        Format the LuminousIntensity to string.
        Note! the default format for LuminousIntensity is Candela.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == LuminousIntensityUnits.Candela:
            return f"""{self.candela} cd"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: LuminousIntensityUnits = LuminousIntensityUnits.Candela) -> str:
        """
        Get LuminousIntensity unit abbreviation.
        Note! the default abbreviation for LuminousIntensity is Candela.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == LuminousIntensityUnits.Candela:
            return """cd"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MagneticFluxUnits(Enum):
        """
            MagneticFluxUnits enumeration
        """
        
        Weber = 'weber'
        """
            
        """
        

class MagneticFlux(AbstractMeasure):
    """
    In physics, specifically electromagnetism, the magnetic flux through a surface is the surface integral of the normal component of the magnetic field B passing through that surface.

    Args:
        value (float): The value.
        from_unit (MagneticFluxUnits): The MagneticFlux unit to create from, The default unit is Weber
    """
    def __init__(self, value: float, from_unit: MagneticFluxUnits = MagneticFluxUnits.Weber):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__webers = None
        

    def convert(self, unit: MagneticFluxUnits) -> float:
        return self.__convert_from_base(unit)

    def __convert_from_base(self, from_unit: MagneticFluxUnits) -> float:
        value = self._value
        
        if from_unit == MagneticFluxUnits.Weber:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MagneticFluxUnits) -> float:
        
        if to_unit == MagneticFluxUnits.Weber:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_webers(webers: float):
        """
        Create a new instance of MagneticFlux from a value in webers.

        

        :param meters: The MagneticFlux value in webers.
        :type webers: float
        :return: A new instance of MagneticFlux.
        :rtype: MagneticFlux
        """
        return MagneticFlux(webers, MagneticFluxUnits.Weber)

    
    @property
    def webers(self) -> float:
        """
        
        """
        if self.__webers != None:
            return self.__webers
        self.__webers = self.__convert_from_base(MagneticFluxUnits.Weber)
        return self.__webers

    
    def to_string(self, unit: MagneticFluxUnits = MagneticFluxUnits.Weber) -> str:
        """
        Format the MagneticFlux to string.
        Note! the default format for MagneticFlux is Weber.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == MagneticFluxUnits.Weber:
            return f"""{self.webers} Wb"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MagneticFluxUnits = MagneticFluxUnits.Weber) -> str:
        """
        Get MagneticFlux unit abbreviation.
        Note! the default abbreviation for MagneticFlux is Weber.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MagneticFluxUnits.Weber:
            return """Wb"""
        


from __future__ import annotations

from functools import total_ordering


@total_ordering
class AbstractMeasure:
    _value: float

    def __str__(self):
        return self.to_string()

    def __add__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value + other._value)

    def __mul__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value * other._value)

    def __sub__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value - other._value)

    def __truediv__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value / other._value)

    def __mod__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value % other._value)

    def __pow__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return type(self)(self._value ** other._value)

    def __eq__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return self._value == other._value

    def __lt__(self, other: AbstractMeasure):
        if not isinstance(other, type(self)):
            return NotImplemented
        return self._value < other._value
